#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>

#include <errno.h>
#include <netdb.h>
#include <stdio.h>
#include <string.h>

#include <openssl/ssl.h>
#include <openssl/bio.h>
#include <openssl/err.h>

#define PORT 2053

#define SERVER 1
static int
makesock(int port)
{
	/* voir en annexe */
}

int /*@ \label{ds:main:beg} @*/
main()
{
	SSL_library_init();/*@ \label{ds:init:beg} @*/
	SSL_load_error_strings();

	FILE* paramfile = fopen("dh_param_1024.pem", "r");
	if (paramfile == NULL) {
		fprintf(stderr, "Error opening the DH file: %s\n",
			strerror(errno));
		return EXIT_FAILURE;
	}

	DH* dh_1024 = PEM_read_DHparams(paramfile, NULL, NULL, NULL);
	fclose(paramfile);
	if (dh_1024 == NULL) {
		fprintf(stderr, "Error reading the DH file\n");
		return EXIT_FAILURE;
	}/*@ \label{ds:init:end} @*/

	int sock = makesock(PORT); /*@ \label{ds:makesock} @*/

	BIO* conn = BIO_new_dgram(sock, BIO_NOCLOSE);/*@ \label{ds:biocr:beg} @*/
	if (conn == NULL) {
		fprintf(stderr,"error creating bio\n");
		return EXIT_FAILURE;
	}/*@ \label{ds:biocr:end} @*/

	SSL_CTX *ctx = SSL_CTX_new(DTLSv1_server_method());/*@ \label{ds:ctxcr:beg} @*/
	if (ctx == NULL) {
		ERR_print_errors_fp(stderr);
		return EXIT_FAILURE;
	}
	SSL_CTX_set_read_ahead(ctx, 1);/*@ \label{ds:ctxcr:rdahd} @*/
	SSL_CTX_set_cipher_list(ctx, "HIGH:MEDIUM:aNULL");/*@ \label{ds:ctxcr:ciphers} @*/
	SSL_CTX_set_tmp_dh(ctx, dh_1024);/*@ \label{ds:ctxcr:end} @*/

	SSL *ssl = SSL_new(ctx); /*@ \label{ds:sslcr:beg} @*/
	if (ssl == NULL) {
		return EXIT_FAILURE;
	}

	SSL_set_bio(ssl, conn, conn);/*@ \label{ds:sslcr:bio} @*/
	SSL_set_accept_state(ssl);/*@ \label{ds:sslcr:end} @*/

	char buf[200] = {0};/*@ \label{ds:ssluse:beg} @*/

	int err = SSL_read(ssl, &buf, 199);
	if (err <= 0) {
		err = SSL_get_error(ssl, err);
		fprintf(stderr, "SSL_read: error %d\n", err);
		ERR_print_errors_fp(stderr);
		return EXIT_FAILURE;
	}
	fprintf(stderr, "%s", buf);

	char *c = "Hello yourself!\n";
	err = SSL_write(ssl, c, strlen(c));
	if (err <= 0) {
		err = SSL_get_error(ssl, err);
		fprintf(stderr, "SSL_write: error %d\n", err);
		ERR_print_errors_fp(stderr);
		if (err == SSL_ERROR_SYSCALL)
			fprintf(stderr, "errno: %s\n", strerror(errno));
	}/*@ \label{ds:ssluse:end} @*/

	SSL_free(ssl); /* free conn too *//*@ \label{ds:clean:beg} @*/
	SSL_CTX_free(ctx);
	DH_free(dh_1024);
	close(sock);/*@ \label{ds:clean:end} @*/
	return 0;
}
